<?php

declare(strict_types=1);

namespace Dedi\SyliusMarketingPlugin\EventListener;

use Dedi\SyliusMarketingPlugin\Factory\RenderHeadTwigFactory;
use Symfony\Component\HttpKernel\Event\ResponseEvent;

class RenderViewListener
{
    private RenderHeadTwigFactory $renderTwig;
    public function __construct(RenderHeadTwigFactory $renderTwig)
    {
        $this->renderTwig = $renderTwig;
    }

    public function onKernelResponse(ResponseEvent $event): void
    {
        if (!$this->isAllowed($event)) {
            return;
        }

        $event->getResponse()->setContent(
            preg_replace(
                '/<head\b[^>]*>/',
                '$0' . $this->renderTwig->render(),
                (string) $event->getResponse()->getContent(),
                1,
            ),
        );
    }

    private function isAllowed(ResponseEvent $event): bool
    {
        $contentType = $event->getResponse()->headers->get('content-type');

        return in_array($contentType, ['text/html', null], true) &&
            null === $event->getResponse()->headers->get('location') &&
            $event->isMainRequest();
    }
}
